/*****************************************************************************

  FileName:     5ESSStateTE.c

  Contents:     AT&T 5ESS ISDN State Engine for TE (User Mode).

	            The controlling state engine for Q.931 is the state engine
	            on the NT side. The state engine on the TE side is a slave 
	            of this. The TE side maintain it's own states as described in
	            ITU-T Q931, but will in    raise conditions be overridden by 
	            the NT side.

	            This reference implementation uses a process per message, 
	            meaning that each message must check call states. This
	            is easier for dialect maintenance as each message proc
	            can be replaced individually. A new TE variant only
	            need to copy the Q931CreateTE and replace those procs or
	            need to override.

  License/Copyright:

  Copyright (c) 2007, Jan Vidar Berger, Case Labs, Ltd. All rights reserved.
  email:janvb@caselaboratories.com  

  Copyright (c) 2007, Michael Jerris. All rights reserved.
  email:mike@jerris.com  
  
  Copyright (c) 2007, Michael S. Collins, All rights reserved.
  email:mcollins@fcnetwork.com
  
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are 
  met:

	* Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
	* Neither the name of the Case Labs, Ltd nor the names of its contributors 
	may be used to endorse or promote products derived from this software 
	without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

#define Q931PRIVATE

#include "5ESS.h"

extern L3INT Q931L4HeaderSpace;
extern struct Q931MessageIE *ATT5ESSMessageIEs;

/*
 * ProtDisc 3 (Service/ServiceAck) and 8 (Connect/ConnectAck) share the same message IDs
 * Use custom functions to handle them
 */
static L3INT ATT5ESSProc0x07TE(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom);
static L3INT ATT5ESSProc0x0fTE(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom);


void ATT5ESSCreateTE(struct Q931Dialect *dlct)
{
	Q931DialectSetName(dlct, "5ESS TE");

	/*
	 * Flags
	 */
	Q931DialectSetFlag(dlct, 0);

	/*
	 * Codesets
	 */
	Q931DialectAddCodeset(dlct, Q931_CODESET_0);
	Q931DialectAddCodeset(dlct, Q931_CODESET_6);
	Q931DialectAddCodeset(dlct, Q931_CODESET_7);

	/*
	 *
	 */
	Q931DialectSetMesProc(dlct, Q931mes_ALERTING,             Q931ProcAlertingTE,          Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CALL_PROCEEDING,      Q931ProcCallProceedingTE,    Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CONNECT,              ATT5ESSProc0x0fTE,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CONNECT_ACKNOWLEDGE,  ATT5ESSProc0x07TE,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_PROGRESS,             Q931ProcProgressTE,          Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SETUP,                Q931ProcSetupTE,             Q931Umes_Generic, Q931Pmes_Generic);
#if 0
	Q931DialectSetMesProc(dlct, Q931mes_SETUP_ACKNOWLEDGE,    Q931ProcSetupAckTE,          Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESUME,               Q931ProcResumeTE,            Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESUME_ACKNOWLEDGE,   Q931ProcResumeAckTE,         Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESUME_REJECT,        Q931ProcResumeRejectTE,      Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SUSPEND,              Q931ProcSuspendTE,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SUSPEND_ACKNOWLEDGE,  Q931ProcSuspendAckTE,        Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SUSPEND_REJECT,       Q931ProcSuspendRejectTE,     Q931Umes_Generic, Q931Pmes_Generic);
#endif
	Q931DialectSetMesProc(dlct, Q931mes_USER_INFORMATION,     Q931ProcUserInformationTE,   Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_DISCONNECT,           Q931ProcDisconnectTE,        Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RELEASE,              Q931ProcReleaseTE,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RELEASE_COMPLETE,     Q931ProcReleaseCompleteTE,   Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESTART,              Q931ProcRestartTE,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESTART_ACKNOWLEDGE,  Q931ProcRestartAckTE,        Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CONGESTION_CONTROL,   Q931ProcCongestionControlTE, Q931Umes_Generic, Q931Pmes_Generic);
#if 0
	Q931DialectSetMesProc(dlct, Q931mes_INFORMATION,          Q931ProcInformationTE,       Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_NOTIFY,               Q931ProcNotifyTE,            Q931Umes_Generic, Q931Pmes_Generic);
#endif
	Q931DialectSetMesProc(dlct, Q931mes_STATUS,               Q931ProcStatusTE,            Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_STATUS_ENQUIRY,       Q931ProcStatusEnquiryTE,     Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SEGMENT,              Q931ProcSegmentTE,           Q931Umes_Generic, Q931Pmes_Generic);

	Q931DialectSetMesProc(dlct, Q932mes_FACILITY,             Q932ProcFacilityTE,          Q932Umes_Facility,       Q932Pmes_Facility);
	Q931DialectSetMesProc(dlct, Q932mes_HOLD,                 Q932ProcHoldTE,              Q932Umes_Hold,           Q932Pmes_Hold);
	Q931DialectSetMesProc(dlct, Q932mes_HOLD_ACKNOWLEDGE,     Q932ProcHoldAckTE,           Q932Umes_HoldAck,        Q932Pmes_HoldAck);
	Q931DialectSetMesProc(dlct, Q932mes_HOLD_REJECT,          Q932ProcHoldRejectTE,        Q932Umes_HoldReject,     Q932Pmes_HoldReject);
	Q931DialectSetMesProc(dlct, Q932mes_REGISTER,             Q932ProcRegisterTE,          Q932Umes_Register,       Q932Pmes_Register);
	Q931DialectSetMesProc(dlct, Q932mes_RETRIEVE,             Q932ProcRetrieveTE,          Q932Umes_Retrieve,       Q932Pmes_Retrieve);
	Q931DialectSetMesProc(dlct, Q932mes_RETRIEVE_ACKNOWLEDGE, Q932ProcRetrieveAckTE,       Q932Umes_RetrieveAck,    Q932Pmes_RetrieveAck);
	Q931DialectSetMesProc(dlct, Q932mes_RETRIEVE_REJECT,      Q932ProcRetrieveRejectTE,    Q932Umes_RetrieveReject, Q932Pmes_RetrieveReject);

	/*
	 * Unknown / Invalid (Unexpected) Message handler
	 */
	Q931DialectSetUnknownMesProc(dlct, Q931ProcUnknownMessage);
	Q931DialectSetInvalidMesProc(dlct, Q931ProcUnexpectedMessage);


	/*
	 * Set up the IE encoder/decoder handle table.
	 */ 
	Q931DialectSetIEProc(dlct, Q931ie_SEGMENTED_MESSAGE,                Q931Pie_Segment,       Q931Uie_Segment);
	Q931DialectSetIEProc(dlct, Q931ie_BEARER_CAPABILITY,                Q931Pie_BearerCap,     Q931Uie_BearerCap);
	Q931DialectSetIEProc(dlct, Q931ie_CAUSE,                            Q931Pie_Cause,         Q931Uie_Cause);
	Q931DialectSetIEProc(dlct, Q931ie_CALL_IDENTITY,                    Q931Pie_CallID,        Q931Uie_CallID);
	Q931DialectSetIEProc(dlct, Q931ie_CALL_STATE,                       Q931Pie_CallState,     Q931Uie_CallState);
	Q931DialectSetIEProc(dlct, Q931ie_CHANGE_STATUS,                    Q931Pie_ChangeStatus,  Q931Uie_ChangeStatus);
	Q931DialectSetIEProc(dlct, Q931ie_CHANNEL_IDENTIFICATION,           Q931Pie_ChanID,        Q931Uie_ChanID);
	Q931DialectSetIEProc(dlct, Q931ie_PROGRESS_INDICATOR,               Q931Pie_ProgInd,       Q931Uie_ProgInd);
	Q931DialectSetIEProc(dlct, Q931ie_NETWORK_SPECIFIC_FACILITIES,      Q931Pie_NetFac,        Q931Uie_NetFac);
	Q931DialectSetIEProc(dlct, Q931ie_NOTIFICATION_INDICATOR,           Q931Pie_NotifInd,      Q931Uie_NotifInd);
	Q931DialectSetIEProc(dlct, Q931ie_DISPLAY,                          Q931Pie_Display,       Q931Uie_Display);
	Q931DialectSetIEProc(dlct, Q931ie_DATETIME,                         Q931Pie_DateTime,      Q931Uie_DateTime);
	Q931DialectSetIEProc(dlct, Q931ie_KEYPAD_FACILITY,                  Q931Pie_KeypadFac,     Q931Uie_KeypadFac);
	Q931DialectSetIEProc(dlct, Q931ie_SIGNAL,                           Q931Pie_Signal,        Q931Uie_Signal);
	Q931DialectSetIEProc(dlct, Q931ie_TRANSIT_DELAY_SELECTION_AND_IND,  Q931Pie_TransNetSel,   Q931Uie_TransNetSel);
	Q931DialectSetIEProc(dlct, Q931ie_CALLING_PARTY_NUMBER,             Q931Pie_CallingNum,    Q931Uie_CallingNum);
	Q931DialectSetIEProc(dlct, Q931ie_CALLING_PARTY_SUBADDRESS,         Q931Pie_CallingSub,    Q931Uie_CallingSub);
	Q931DialectSetIEProc(dlct, Q931ie_CALLED_PARTY_NUMBER,              Q931Pie_CalledNum,     Q931Uie_CalledNum);
	Q931DialectSetIEProc(dlct, Q931ie_CALLED_PARTY_SUBADDRESS,          Q931Pie_CalledSub,     Q931Uie_CalledSub);
	Q931DialectSetIEProc(dlct, Q931ie_TRANSIT_NETWORK_SELECTION,        Q931Pie_TransNetSel,   Q931Uie_TransNetSel);
	Q931DialectSetIEProc(dlct, Q931ie_RESTART_INDICATOR,                Q931Pie_RestartInd,    Q931Uie_RestartInd);
	Q931DialectSetIEProc(dlct, Q931ie_LOW_LAYER_COMPATIBILITY,          Q931Pie_LLComp,        Q931Uie_LLComp);
	Q931DialectSetIEProc(dlct, Q931ie_HIGH_LAYER_COMPATIBILITY,         Q931Pie_HLComp,        Q931Uie_HLComp);
	Q931DialectSetIEProc(dlct, Q931ie_USER_USER,                        Q931Pie_UserUser,      Q931Uie_UserUser);
	Q931DialectSetIEProc(dlct, Q931ie_GENERIC_DIGITS,                   Q931Pie_GenericDigits, Q931Uie_GenericDigits);

	Q931DialectSetIEProc(dlct, Q931ie_CONNECTED_NUMBER,                 Q931Pie_Generic,       Q931Uie_Generic);
	Q931DialectSetIEProc(dlct, Q931ie_FACILITY,                         Q931Pie_Generic,       Q931Uie_Generic);


	/*
	 * State Table
	 */

	/*
	 * Q.931 message <-> ie table
	 */
	Q931DialectSetMesIEMap(dlct, ATT5ESSMessageIEs);
}

/*****************************************************************************

  Function:		ATT5ESSProc0x0fTE

*****************************************************************************/
static L3INT ATT5ESSProc0x0fTE(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom)
{
	Q931mes_Header *pMes = (Q931mes_Header *)&buf[Q931L4HeaderSpace];
	L3INT ret = Q931E_NO_ERROR;

	switch (pMes->ProtDisc) {
	case 8:
		return Q931ProcConnectAckTE(pTrunk, buf, iFrom);

	case 3:
		if (iFrom == 4) {
			/* TODO Add proc here*/
			ret = Q931Tx32Data(pTrunk, 0, buf, pMes->Size);
		}
		else if (iFrom == 2) {
			/* TODO Add proc here*/
			ret = Q931Tx34(pTrunk, buf, pMes->Size);

			if (Q931TrunkIsSetFlag(pTrunk, Q931_TFLAG_AUTO_SERVICE_ACK)) {
				Q931Log(pTrunk, Q931_LOG_DEBUG, "autoServiceAck is on, responding to Service Req from network...\n");
				Q931AckService(pTrunk, buf);
			}
		}
		return ret;

	default:
		break;
	}
	return Q931E_NO_ERROR;
}

/*****************************************************************************

  Function:		ATT5ESSProc0x07TE

*****************************************************************************/
static L3INT ATT5ESSProc0x07TE(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom)
{
	Q931mes_Header *pMes = (Q931mes_Header *)&buf[Q931L4HeaderSpace];
	L3INT ret = Q931E_NO_ERROR;

	switch (pMes->ProtDisc) {
	case 8:
		return Q931ProcConnectTE(pTrunk, buf, iFrom);

	case 3:
		if (iFrom == 4) {
			/* TODO Add proc here*/
			ret = Q931Tx32Data(pTrunk, 0, buf, pMes->Size);
		}
		else if (iFrom == 2) {
			/* TODO Add proc here*/
			ret = Q931Tx34(pTrunk, buf, pMes->Size);
		}
		return ret;

	default:
		break;
	}
	return Q931E_NO_ERROR;
}
